﻿/**
 * Api
 * 
 * The use of a common API fulfills many of the SCORM’s high-level requirements for interoperability and reuse.
 * It provides a standardized way for SCOs to communicate with LMSs, yet it shields the particular communication implementation from the content developer.
 * In its simplest terms, an API is merely a set of predefined functions that the SCO can rely on being available.
 * An API hides implementation details from SCOs and thus promotes reuse and interoperability.
 * An API Adapter is a piece of functional software that implements and exposes the functions of the API.
 * How the insides of an API Adapter are implemented should not matter to content developers provided they use the same public interface.
 * The LMS need only provide an API Adapter that implements the functionality of the API and exposes its interface to the client SCO.
 * A key aspect of the API is that it is a communication mechanism that allows the SCO to communicate with the LMS.
 * It is assumed that once the SCO is launched it can then "get" and "set" information with an LMS.
 * All communication between the API Adapter and the SCO is initiated by the SCO.
 * There is currently no supported mechanism for LMSs to initiate calls to functions implemented by a SCO.
 * 
 * The functions of the API Adapter object are threefold:
 * 
 * - Execution State
 * Two of the API functions, LMSInitialize("") and LMSFinish(""), handle execution state.
 * 
 * - State Management
 * The API has three functions that are used to handle errors.
 * These three API functions are: LMSGetLastError(), LMSGetErrorString(errornumber) and LMSGetDiagnostic(parameter).
 * 
 * - Data Transfer
 * The remaining three API functions are used to transfer data to and from an LMS:
 * LMSGetValue(data model element), LMSSetValue(data model element, value) and LMSCommit("")
 *
 * $Project: CLX.WBT-Tracker$
 * $Date: 12.07.2011 17:21:37$ 
 * $Revision: 7$
 *
 * @package    	clx.scorm
 * @copyright  	Copyright (c) Crealogix AG
 * @link		www.crealogix.com
 * @since       1.0.0
 * @author      Andreas HÄberli <andreas.haeberli@crealogix.com>
 * @version		$Id: Api.js,v 1.2.1.2.1.0, 2011-07-12 15:21:37Z, Andreas Haeberli$
 */
 
dojo.require('clx.scorm.Behavior');
dojo.require('clx.scorm.Error');	

dojo.provide('clx.scorm.Api');
dojo.declare('clx.scorm.Api',null,{
	
	constructor: function(storage) {
		// console.info('SCORM 1.2 API');		
		// console.info(storage.toInfo());
		this._isInitialize = false;		
		this._error = new clx.scorm.Error('0');
    	storage.register(this);
		this._behavior = new clx.scorm.Behavior(storage);
		dojo.connect(this._behavior, 'onInitializeData', this, 'onLoad');
		this.addons = [];
	},
	
	/*
	 * LMSInitialize
	 * This function indicates to the API Adapter that the SCO is going to communicate with the LMS.
	 * It allows the LMS to handle LMS specific initialization issues.
	 * It is a requirement of the SCO that it call this function before calling any other API functions.
	 * 
	 * @param string parameter
	 * @return string
	 */		
	LMSInitialize:function(parameter) {
		// console.info('LMSInitialize('+parameter+') has been invoked');
		if(parameter == '') {
			// console.info('-> Parameter: '+parameter);
			if(this._isInitialize) {
				this._error = new clx.scorm.Error('0');			
				// console.log('--> true ('+this._error.getCode()+' | LMSInitialize already invoked)');
				return 'true';
			} else {
				if(this._behavior.initializeData()) {					
					this._isInitialize = true;
					this._error = new clx.scorm.Error('0');
					// console.log('--> true ('+this._error.getCode()+' | LMSInitialize finished successfully)');
					return 'true';
				} else {		
					this._error = new clx.scorm.Error('301');
					// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
					return 'false';
				}
			}
		} else {			
			this._error = new clx.scorm.Error('201');
			// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+': "'+parameter+'")');	
			return 'false';
		}
	},
	
	/*
	 * LMSFinish
	 * The SCO must call this when it has determined that it no longer needs to communicate with the LMS,
	 * if it successfully called LMSInitialize at any previous point. 
	 * This call signifies two things:
	 * 1. The SCO can be assured that any data set using LMSSetValue() calls has been persisted by the LMS.
	 * 2. The SCO has finished communicating with the LMS.
	 *
	 * @param string parameter
	 * @return string
	 */		
	LMSFinish:function(parameter) {
		// console.info('LMSFinish('+parameter+') has been invoked');
		if(parameter == '') {
			// console.info('-> Parameter: '+parameter);
			if(this._isInitialize) {
				if(this._behavior.finish()) {					
					this._isInitialize = false;
					this._error = new clx.scorm.Error('0');
					// console.log('--> true ('+this._error.getCode()+' | LMSFinish finished successfully)');
					return 'true';
				} else {
					this._error = new clx.scorm.Error('101');
					// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');				
					return 'false';
				}
			} else {
				this._error = new clx.scorm.Error('301');
				// console.warn('--> false ('+this._error.getCode()+' | Not initialized)');
				return 'false';
			}
		} else {			
			this._error = new clx.scorm.Error('201');
			// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+': "'+parameter+'")');	
			return 'false';
		}
	},
	
	/*
	 * LMSGetValue
	 * This function allows the SCO to obtain information from the LMS.
	 * It is used to determine:
	 * - Values for various categories (groups) and elements in the data model
	 * - The version of the data model supported
	 * - Whether a specific category or element is supported
	 * - The number of items currently in an array or list of elements
	 * The complete data element name and/or keywords are provided as a parameter.
	 * The current value of the requested data model parameter is returned.
	 * Only one value -- always a string -- is returned for each call.
	 * 
	 * @param string parameter
	 * @return string
	 */		
	LMSGetValue:function(parameter) {
		// console.info('LMSGetValue('+parameter+') has been invoked');
		if(this._isInitialize) {
			if(parameter != '') {
				// console.info('-> Parameter: '+parameter);			
				var value = this._behavior.getData(parameter);
				if(typeof(value) == 'string') {
					this._error = new clx.scorm.Error('0');
					// console.log('--> "'+value+'" ('+this._error.getCode()+' | LMSGetValue finished successfully)');
					return value;
				} else {
					this._error = value;				
					// console.warn('--> "" ('+this._error.getCode()+' | '+this._error.getMessage()+')');
					return '';
				}
			} else {
				this._error = new clx.scorm.Error('201');
				// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
				return 'false';
			}	
		} else {
			this._error = new clx.scorm.Error('301');
			// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
			return 'false';
		}	
	},
	
	/*
	 * LMSSetValue
	 * This function allows the SCO to send information to the LMS.
	 * The API Adapter may be designed to immediately forward the information to the LMS,
	 * or it may be designed to forward information based on some other approach.
	 * This function is used to set the current values for various categories (groups) and elements in the data model.
	 * The data element name and its group are provided as a parameter.
	 * The newly desired value of the data element is included as the second parameter in the call.
	 * Only one value is sent with each call.
	 * 
	 * @param string parameter
	 * @param string value
	 * @return string
	 */		
	LMSSetValue:function(parameter, value) {
		// console.info('LMSSetValue() has been invoked');
		if(this._isInitialize) {
			if(parameter != '') {
				// console.info('-> Parameter: '+parameter+' / Value:'+value);
				var result = this._behavior.setData(String(parameter), String(value));
				if(typeof(result) == 'string') {
					this._error = new clx.scorm.Error('0');
					// console.log('--> true ('+this._error.getCode()+' | LMSSetValue finished successfully)');
					return 'true';
				} else {
					this._error = result;
					// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
					return 'false';
				}			
			} else {			
				this._error = new clx.scorm.Error('201');
				// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');	
				return 'false';
			}
		} else {
			this._error = new clx.scorm.Error('301');
			// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
			return 'false';
		}
	},
	
	/*
	 * LMSCommit
	 * If the API Adapter is caching values received from the SCO via an LMSSetValue(),
	 * this call requires that any values not yet persisted by the LMS be persisted.
	 * In some implementations, the API Adapter may persist set values as soon as they are received, and not cache them on the client. 
	 * In such implementations, this API call is redundant and would result in no additional action from the API Adapter. 
	 * This call ensures to the SCO that the data sent, via an LMSSetValue() call,
	 * will be persisted by the LMS upon completion of the LMSCommit().
	 * 
	 * @param string parameter
	 * @return string
	 */		
	LMSCommit:function(parameter) {
		// console.info('LMSCommit() has been invoked');
		if(this._isInitialize) {
			if(parameter == '') {
				// console.info('-> Parameter: '+parameter);
				if(this._behavior.commitData()) {
					this._error = new clx.scorm.Error('0');
					// console.log('--> true ('+this._error.getCode()+' | LMSCommit finished successfully)');
					return 'true';
				} else {
					this._error = new clx.scorm.Error('101');
					// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
					return 'false';
				}
			} else {
				this._error = new clx.scorm.Error('201');
				// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');	
				return 'false';
			}	
		} else {
			this._error = new clx.scorm.Error('301');
			// console.warn('--> false ('+this._error.getCode()+' | '+this._error.getMessage()+')');
			return 'false';
		}	
	},
	
	/*
	 * LMSGetLastError
	 * The SCO must have a way of assessing whether or not any given API call was successful,
	 * and if it was not successful, what went wrong. 
	 * This function returns an error status code resulting from the previous API call. 
	 * Each time an API function is called (with the exception of this one, LMSGetErrorString, and LMSGetDiagnostic
	 * -- the error functions), the error code is reset.
	 * The SCO may call the error functions any number of times to retrieve the error code,
	 * and the code cannot change until the next API call is made.
	 * @return int
	 */		
	LMSGetLastError:function() {
		// console.info('LMSGetLastError() has been invoked');
		// console.log('--> '+this._error.getCode()+' | LMSGetLastError finished successfully');
		return this._error.getCode();		
	},
	
	/*
	 * LMSGetErrorString
	 * This function enables the content to obtain a textual description of the error represented by the error code number.
	 * 
	 * @param int errornumber
	 * @return string
	 */		
	LMSGetErrorString:function(errornumber) {
		// console.info('LMSGetErrorString() has been invoked');
		if(errornumber != undefined) {
			// console.info('-> Error number: ('+errornumber+')');
			// console.log('--> '+this._error.getMessage(errornumber)+' | LMSGetLastError finished successfully');	
			return this._error.getMessage(errornumber);
		} else {
			this._error = new clx.scorm.Error('201');
			// console.warn('--> "" ('+this._error.getCode()+' | '+this._error.getMessage()+')');
			return '';			
		}	
	},
	
	/*
	 * LMSGetDiagnostic
	 * This function enables vendor-specific error descriptions to be developed and accessed by the content.
	 * These would normally provide additional detail regarding the error.
	 * 
	 * @param string parameter
	 * @return string
	 */		
	LMSGetDiagnostic:function(parameter) {
		// console.info('LMSGetDiagnostic() has been invoked');
		if(parameter != undefined) {
			// console.info('-> Parameter: '+parameter);
			// console.log('--> '+this._error.getMessage(parameter)+' | LMSGetDiagnostic finished successfully');	
			return error.getMessage(parameter);
		} else {
			this._error = new clx.scorm.Error('201');
			// console.warn('--> "" ('+this._error.getCode()+' | '+this._error.getMessage()+')');	
			return '';			
		}	
	},
	
	/**
	 * This function receives an addon
	 * and adds all functions of this object to the API object,
	 * as long as it doesn't exist yet.
	 * 
	 * @param	object
	 */
	 registerAddon : function(addon){
        this.addons.push(addon);
        
        for(var func in addon){
			addon.register(this);
            if(typeof addon[func] === 'function' && func.indexOf('LMS') != -1){
                if(!this.hasOwnProperty(addon[func])){
                    var that = this;
                    this[func] = (function(funcName, addon){
                        return function(){
                            return addon[funcName].apply(addon, arguments);
                        }
                    })(func, addon);
                }
            }
        }
    },
		
	/**
	 * On load
	 * 
	 * @return string
	 */
	onLoad:function(response){
		// console.info('onLoad() has been invoked');
	}
	
});